clear all
close all
clc

%% Import Data
% Define the file path and filenames for IV and dI/dV (Aux) line scans.

basePath = 'C:\Users\Dennis\OneDrive - University of Twente\PhD\Papers\Nanoribbons\Klaassen Nature Communications\15th version\data\Figure 3\OA\STS line spectra\NR 5 nm';

filenameAux = '\Aux1(V) (STS_LOCATION-220,153;4_94667e-009,2_12e-010) Backward Thu Nov 16 14_30_17 2023 [121-1]  AUX.asc';
filenameIV = '\Aux1(V) (STS_LOCATION-220,153;4_94667e-009,2_12e-010) Backward Thu Nov 16 14_30_17 2023 [121-1]  AUX.asc';

% Construct full file paths
dataPathAux = strcat(basePath, filenameAux);
dataPathIV = strcat(basePath, filenameIV);

% Read and organize data
importAux = importdata(dataPathAux);
importIV = importdata(dataPathIV);

dataAux = importAux.data;
dataIV = importIV.data;

%% Organize Voltage, I(V), and Aux Data
[rownum, colnum] = size(dataAux);
numCurves = colnum / 2;

Xdata = zeros(rownum, numCurves);
Ydata = Xdata;
XdataIV = Xdata;
YdataIV = Xdata;

for jj = 1:numCurves
    colIdx = 2 * (jj - 1);
    Xdata(:, jj) = dataAux(:, colIdx + 1);  % Voltage data
    Ydata(:, jj) = dataAux(:, colIdx + 2);  % Aux data
    
    XdataIV(:, jj) = dataIV(:, colIdx + 1); % Voltage data (IV)
    YdataIV(:, jj) = dataIV(:, colIdx + 2); % I/V data
end

%% Binning of I(V) and Aux curves
binning = 4; % Set bin size, 1 means no binning

numBinnedCurves = numCurves / binning;
YdataBinned = zeros(rownum, numBinnedCurves);
YdataIVBinned = zeros(rownum, numBinnedCurves);

for jj = 1:numBinnedCurves
    binStartIdx = (jj - 1) * binning + 1;
    binEndIdx = jj * binning;
    
    YdataBinned(:, jj) = mean(Ydata(:, binStartIdx:binEndIdx), 2);    % Aux
    YdataIVBinned(:, jj) = mean(YdataIV(:, binStartIdx:binEndIdx), 2); % I/V
end

% Update variables to hold binned data
Ydata = YdataBinned;
YdataIV = YdataIVBinned;
[rownum, colnum] = size(Ydata);
colnum = colnum * 2; % Update column count

%% Data Analysis
close all

smoothedDataY = zeros(size(Ydata));
NumYdataIV = zeros(size(YdataIV, 1) - 1, size(YdataIV, 2));

for ii = 1:size(Ydata, 2)
    smoothedDataY(:, ii) = smooth(Ydata(:, ii), 15) / 1e9; % Smoothing Aux data
    NumYdataIV(:, ii) = smooth(diff(smooth(YdataIV(:, ii), 35)) ./ diff(Xdata(:, 1)), 35); % Numerical differentiation
end

%% Further Processing
x1 = 1;
x2 = 401;
dim = size(smoothedDataY);

smoothedDOS = zeros(size(smoothedDataY));
for jj = 1:dim(2)
    smoothedDOS(:, jj) = smooth(smoothedDataY(:, jj), 1);
end

% Interpolation for better visualization
IMG = interp2(smoothedDOS(1:end, 14:25), 0);

% Additional smoothing in spatial dimension
for i = 1:size(IMG, 1)
    IMG(i, :) = smooth(IMG(i, :), 1);
end

%% Plotting
figure()
imagesc([0 size(IMG, 2)], [Xdata(x1) Xdata(x2)], interp2(log(IMG),1))
c = colorbar;
colormap jet

% Add colorbar label
ylabel(c, 'Log(dI/dV) (arb. units)', 'FontSize', 18)

set(gcf, 'units', 'points', 'position', [25, 35, 1100, 540])
set(gca, 'FontSize', 20)
xlabel('Distance (pixels)')
ylabel('Bias voltage (mV)')
title('dI/dV from Aux')

hold off

%% Save Data and Plot

% Define filenames
outputTxtFile = 'dIdV_map_data_v2.txt';
outputPdfFile = 'dIdV_map_plot_v2.pdf';

% Save plot as PDF
saveas(gcf, outputPdfFile);
disp(['Plot saved to: ', outputPdfFile]);

% Save the source data used in the plot
fid = fopen(outputTxtFile, 'w');

% Write setpoints information
voltageSetpoint = 300; % Example voltage setpoint (modify if needed)
currentSetpoint = 0.2; % Example current setpoint (modify if needed)

fprintf(fid, 'Voltage Setpoint (mV): %.3f\n', voltageSetpoint);
fprintf(fid, 'Current Setpoint (nA): %.3e\n', currentSetpoint);
fprintf(fid, '--------------------------------------\n');

% Write header
fprintf(fid, 'Pixel\tVoltage(mV)\tLog(dI/dV)\n');

% Save the source data used for the plot with voltage first
for voltageIdx = 1:size(Xdata, 1)
    for pixelIdx = 1:size(smoothedDOS, 2)
        fprintf(fid, '%d\t%.6f\t%.6f\n', pixelIdx, Xdata(voltageIdx, 1), log(smoothedDOS(voltageIdx, pixelIdx)));
    end
end

fclose(fid);
disp(['Source data saved to: ', outputTxtFile]);

% Save matrices in MATLAB format for further processing
save('dIdV_source_data_v2.mat', 'Xdata', 'Ydata', 'YdataIV', 'IMG', 'smoothedDOS');

disp('Processing, visualization, and data saving complete.');

%%
% Define voltage and current setpoints
voltageSetpoint = 300;  % Example voltage setpoint in millivolts
currentSetpoint = 0.2;  % Example current setpoint in nanoamperes

% Create figure and hold
figure()
hold on

% Plot the data
plot(Xdata(1:end,1), (IMG(:,10)), 'r')  % Plot column 10 of IMG (end)
plot(Xdata(1:end,1), (IMG(:,7)), 'k')   % Plot column 7 of IMG (bulk)

% Set axis limits
xlim([-300 300])

% Add legend and labels
legend('end', 'bulk')
box on
axis square

xlabel('Voltage (mV)')
ylabel('AuX1 (arb. u.)')

% Save the plot as PDF
outputPdfFile = 'plot_voltage_vs_Aux1.pdf';
saveas(gcf, outputPdfFile);
disp(['Plot saved to: ', outputPdfFile]);

% Save the data to a text file
outputTxtFile = 'plot_data_voltage_vs_Aux1.txt';
fid = fopen(outputTxtFile, 'w');

% Write setpoints information
fprintf(fid, 'Voltage Setpoint (mV): %.3f\n', voltageSetpoint);
fprintf(fid, 'Current Setpoint (nA): %.3e\n', currentSetpoint);
fprintf(fid, '--------------------------------------\n');

% Write header
fprintf(fid, 'Voltage(mV)\tEnd_Aux1\tBulk_Aux1\n');

% Write data
for i = 1:length(Xdata(:,1))
    fprintf(fid, '%.6f\t%.6f\t%.6f\n', Xdata(i,1), IMG(i,10), IMG(i,7));
end

% Close file
fclose(fid);
disp(['Data saved to: ', outputTxtFile]);

disp('Processing and saving completed.');








