clear all
close all
clc

%% Import Data
% Define the file path and filenames for IV and dI/dV (Aux) line scans.

basePath = 'C:\Users\Dennis\OneDrive - University of Twente\PhD\Papers\Nanoribbons\Klaassen Nature Communications\15th version\data\Figure 3\OA\STS line spectra\NR 5 nm';

filenameAux = '\Aux1(V) (STS_LOCATION-220,153;4_94667e-009,2_12e-010) Backward Thu Nov 16 15_10_19 2023 [221-1]  AUX.asc';
filenameIV = filenameAux;

% Construct full file paths
dataPathAux = strcat(basePath, filenameAux);
dataPathIV = strcat(basePath, filenameIV);

% Read and organize data
importAux = importdata(dataPathAux);
importIV = importdata(dataPathIV);

dataAux = importAux.data;
dataIV = importIV.data;

% Organize Voltage, I(V), and Aux Data
[rownum, colnum] = size(dataAux);
numCurves = colnum / 2;

XdataAux = zeros(rownum, numCurves);
YdataAux = XdataAux;
XdataIV = XdataAux;
YdataIV = XdataAux;

for jj = 1:numCurves
    colIdx = 2 * (jj - 1);
    XdataAux(:, jj) = dataAux(:, colIdx + 1); % Voltage data
    YdataAux(:, jj) = dataAux(:, colIdx + 2); % Aux data
    
    XdataIV(:, jj) = dataIV(:, colIdx + 1); % Voltage data (IV)
    YdataIV(:, jj) = dataIV(:, colIdx + 2); % I/V data
end

% Binning of I(V) and Aux curves
binning = 2; % Set bin size, 1 means no binning

numBinnedCurves = numCurves / binning;
YdataAuxBinned = zeros(rownum, numBinnedCurves);
YdataIVBinned = zeros(rownum, numBinnedCurves);

for jj = 1:numBinnedCurves
    binStartIdx = (jj - 1) * binning + 1;
    binEndIdx = jj * binning;
    
    YdataAuxBinned(:, jj) = mean(YdataAux(:, binStartIdx:binEndIdx), 2);
    YdataIVBinned(:, jj) = mean(YdataIV(:, binStartIdx:binEndIdx), 2);
end

% Update variables to hold binned data
YdataAux = YdataAuxBinned;
YdataIV = YdataIVBinned;
[rownum, colnum] = size(YdataAux);
colnum = colnum * 2; % Update column count

%% Data Analysis
close all

smoothedDataAux = zeros(size(YdataAux));
dIdV = zeros(size(YdataIV, 1) - 1, size(YdataIV, 2));

for ii = 1:size(YdataAux, 2)
    smoothedDataAux(:, ii) = smooth(YdataAux(:, ii), 1) / 1e9; % Smoothing Aux data
    dIdV(:, ii) = smooth(diff(YdataIV(:, ii)) ./ diff(XdataAux(:, 1)), 35); % Numerical differentiation
end

% Further Processing

x1 = 1;
x2 = 401;
smoothedDOS = zeros(size(smoothedDataAux));

for jj = 1:size(smoothedDataAux, 2)
    smoothedDOS(:, jj) = smooth(smoothedDataAux(:, jj), 5);
end

% Interpolation for better visualization
IMG = interp2(smoothedDOS(x1:x2, :), 0);

% Additional smoothing in spatial dimension
for i = 1:size(IMG, 1)
    IMG(i, :) = smooth(IMG(i, :), 3);
end

%% Plotting
figure()
imagesc([0 size(IMG, 2)], [XdataAux(x1) XdataAux(x2)], log(IMG))
c = colorbar;
colormap jet
caxis([-1 1])

% Add colorbar label
ylabel(c, 'Log(dI/dV) (arb. units)', 'FontSize', 18)

set(gcf, 'units', 'points', 'position', [25, 35, 1100, 540])
set(gca, 'FontSize', 20)
xlabel('Distance (pixels)')
ylabel('Bias voltage (mV)')
title('dI/dV from Aux')

hold off

%% Save Data and Plot

% Define filenames
outputTxtFile = 'dIdV_map_data.txt';
outputPdfFile = 'dIdV_map_plot.pdf';

% Save plot as PDF
saveas(gcf, outputPdfFile);
disp(['Plot saved to: ', outputPdfFile]);

% Prepare and save data to a text file
fid = fopen(outputTxtFile, 'w');

% Write setpoints information
voltageSetpoint = 300; % Example: take the max voltage as a setpoint
currentSetpoint = 0.2; % Example: take the max current as a setpoint

fprintf(fid, 'Voltage Setpoint (mV): %.3f\n', voltageSetpoint);
fprintf(fid, 'Current Setpoint (nA): %.3e\n', currentSetpoint);
fprintf(fid, '--------------------------------------\n');

% Write header
fprintf(fid, 'Pixel\tVoltage(mV)\tLog(dI/dV)\n');

% Write data to file
for i = 1:size(IMG, 1)
    for j = 1:size(IMG, 2)
        fprintf(fid, '%d\t%.6f\t%.6f\n', j, XdataAux(i, 1), log(IMG(i, j)));
    end
end

fclose(fid);
disp(['Data saved to: ', outputTxtFile]);

disp('Processing and visualization complete.');
